@@ -3,12 +3,20 @@
 #include "vulkan_mock.hpp"
 #include "vulkan_mock_setup.hpp"
 
+// Because vulkan-hpp isn't clean of warnings, disable them while including it
+#if defined(__clang__) || defined(__GNUC__) || defined(__GNUG__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wconversion"
+#endif
+#define VULKAN_HPP_NO_SPACESHIP_OPERATOR 
 #include <vulkan/vulkan.hpp>
-
+#if defined(__clang__) || defined(__GNUC__) || defined(__GNUG__)
+#pragma GCC diagnostic pop
+#endif
 
 TEST_CASE("VulkanHpp Instance with surface", "[VkBootstrap.vulkan_hpp]") {
     VulkanMock& mock = get_and_setup_default();
-    mock.api_version = VK_API_VERSION_1_1;
+    mock.instance_api_version = VK_API_VERSION_1_1;
     mock.physical_devices_details[0].properties.apiVersion = VK_API_VERSION_1_1;
     mock.physical_devices_details[0].extensions.push_back(get_extension_properties("VK_KHR_multiview"));
     mock.physical_devices_details[0].extensions.push_back(get_extension_properties("VK_KHR_driver_properties"));
@@ -18,13 +26,13 @@ TEST_CASE("VulkanHpp Instance with surface", "[VkBootstrap.vulkan_hpp]") {
     physical_device_descriptor_indexing_features.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_DESCRIPTOR_INDEXING_FEATURES;
     physical_device_descriptor_indexing_features.runtimeDescriptorArray = true;
     physical_device_descriptor_indexing_features.descriptorBindingPartiallyBound = true;
-    mock.physical_devices_details[0].add_features_pNext_struct(physical_device_descriptor_indexing_features);
+    mock.physical_devices_details[0].features_pNextChain.emplace_back(create_serialized_struct_from_object(physical_device_descriptor_indexing_features));
     mock.physical_devices_details[0].extensions.push_back(get_extension_properties("VK_EXT_subgroup_size_control"));
     VkPhysicalDeviceSubgroupSizeControlFeatures physical_device_subgroup_size_control_features{};
     physical_device_subgroup_size_control_features.sType = VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_SUBGROUP_SIZE_CONTROL_FEATURES;
     physical_device_subgroup_size_control_features.subgroupSizeControl = true;
     physical_device_subgroup_size_control_features.computeFullSubgroups = true;
-    mock.physical_devices_details[0].add_features_pNext_struct(physical_device_subgroup_size_control_features);
+    mock.physical_devices_details[0].features_pNextChain.emplace_back(create_serialized_struct_from_object(physical_device_subgroup_size_control_features));
 
     auto surface = mock.get_new_surface(get_basic_surface_details());
     vk::SurfaceKHR hpp_surface{ surface };
