@@ -2,8 +2,6 @@
 
 #include "vulkan_mock.hpp"
 
-#include <cstring>
-
 #include <algorithm>
 
 #define GPA_IMPL(x)                                                                                                    \
@@ -50,7 +48,7 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkEnumerateInstanceVersion(uint32_t* pApiVer
     if (pApiVersion == nullptr) {
         return VK_ERROR_DEVICE_LOST;
     }
-    *pApiVersion = mock.api_version;
+    *pApiVersion = mock.instance_api_version;
     return VK_SUCCESS;
 }
 
@@ -72,13 +70,16 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkEnumerateInstanceLayerProperties(uint32_t*
     return fill_out_count_pointer_pair(mock.instance_layers, pPropertyCount, pProperties);
 }
 
-VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateInstance([[maybe_unused]] const VkInstanceCreateInfo* pCreateInfo,
-    [[maybe_unused]] const VkAllocationCallbacks* pAllocator,
-    VkInstance* pInstance) {
+VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateInstance(
+    const VkInstanceCreateInfo* pCreateInfo, [[maybe_unused]] const VkAllocationCallbacks* pAllocator, VkInstance* pInstance) {
     if (pInstance == nullptr) {
         return VK_ERROR_INITIALIZATION_FAILED;
     }
     *pInstance = get_handle<VkInstance>(0x0000ABCDU);
+
+    if (pCreateInfo && pCreateInfo->pApplicationInfo && mock.should_save_api_version) {
+        mock.api_version_set_by_vkCreateInstance = pCreateInfo->pApplicationInfo->apiVersion;
+    }
     return VK_SUCCESS;
 }
 VKAPI_ATTR void VKAPI_CALL shim_vkDestroyInstance(
@@ -185,7 +186,7 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateDevice(VkPhysicalDevice physicalDevi
     if (pCreateInfo->pEnabledFeatures) {
         new_feats = *pCreateInfo->pEnabledFeatures;
     }
-    std::vector<std::vector<char>> new_chain;
+    std::vector<SerializedStruct> new_chain;
     std::vector<const char*> created_extensions;
     for (uint32_t i = 0; i < pCreateInfo->enabledExtensionCount; i++) {
         created_extensions.push_back(pCreateInfo->ppEnabledExtensionNames[i]);
@@ -194,11 +195,9 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateDevice(VkPhysicalDevice physicalDevi
     while (pNext_chain) {
         const auto* chain = static_cast<const VkBaseOutStructure*>(pNext_chain);
         const void* next = chain->pNext;
-        if (check_if_features2_struct(chain->sType) > 0) {
-            std::vector<char> new_node;
-            new_node.resize(get_pnext_chain_struct_size(chain->sType));
-            std::memcpy(new_node.data(), pNext_chain, new_node.size());
-            new_chain.push_back(new_node);
+        SerializedStruct new_struct = create_serialized_struct_from_features2_struct(pNext_chain, chain->sType);
+        if (!new_struct.empty()) {
+            new_chain.emplace_back(new_struct);
         }
         if (chain->sType == VK_STRUCTURE_TYPE_PHYSICAL_DEVICE_FEATURES_2) {
             new_feats = static_cast<const VkPhysicalDeviceFeatures2*>(pNext_chain)->features;
@@ -274,13 +273,24 @@ VKAPI_ATTR VkResult VKAPI_CALL shim_vkCreateImageView([[maybe_unused]] VkDevice
     [[maybe_unused]] const VkImageViewCreateInfo* pCreateInfo,
     [[maybe_unused]] const VkAllocationCallbacks* pAllocator,
     VkImageView* pView) {
+    if (mock.fail_image_creation_on_iteration != std::numeric_limits<uint32_t>::max() &&
+        mock.fail_image_creation_on_iteration == mock.created_image_view_count) {
+        return VK_ERROR_INITIALIZATION_FAILED;
+    }
+
     if (pView) *pView = get_uint64_handle<VkImageView>(0x0000CCCEU);
+    mock.created_image_view_count++;
     return VK_SUCCESS;
 }
 
 VKAPI_ATTR void VKAPI_CALL shim_vkDestroyImageView([[maybe_unused]] VkDevice device,
     [[maybe_unused]] VkImageView imageView,
-    [[maybe_unused]] const VkAllocationCallbacks* pAllocator) {}
+    [[maybe_unused]] const VkAllocationCallbacks* pAllocator) {
+    if (mock.created_image_view_count == 0) {
+        throw std::runtime_error("no created image views to destroy!");
+    }
+    mock.created_image_view_count--;
+}
 
 VKAPI_ATTR void VKAPI_CALL shim_vkDestroySwapchainKHR([[maybe_unused]] VkDevice device,
     [[maybe_unused]] VkSwapchainKHR swapchain,
